package com.iconplus.smartproc.service.authentication;

import com.iconplus.smartproc.configuration.JwtTokenUtil;
import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.entity.TokenManagement;
import com.iconplus.smartproc.model.request.LoginRequest;
import com.iconplus.smartproc.model.response.LoginResponse;
import com.iconplus.smartproc.model.token.TokenContent;
import com.iconplus.smartproc.repository.UsersRepository;
import com.iconplus.smartproc.service.CommonService;
import com.iconplus.smartproc.util.Constants;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

@Service
@Slf4j
public class LoginService implements BaseService<LoginRequest, LoginResponse> {

    @Value("${jwt.expired-time.access-token:15}")
    private Integer accessTokenExp;
    @Value("${jwt.expired-time.refresh-token:60}")
    private Integer refreshTokenExp;

    private final UsersRepository usersRepository;
    private final JwtTokenUtil jwtTokenUtil;
    private final CommonService commonService;


    public LoginService(UsersRepository userRepository,
                        JwtTokenUtil jwtTokenUtil,
                        CommonService commonService) {
        this.usersRepository = userRepository;
        this.jwtTokenUtil = jwtTokenUtil;
        this.commonService = commonService;

    }

    @Transactional
    @Override
    public LoginResponse execute(LoginRequest input) {

        var userRoleView= usersRepository.getByUsernameOrEmail(input.getEmail())
                .orElseThrow(() -> new BusinessException(HttpStatus.CONFLICT,
                        Constants.ERR_CODE_10003,
                        Constants.ERR_TTL_10003,
                        String.format(Constants.ERR_MSG_10003, input.getEmail())));

        if (!StringUtils.equalsIgnoreCase(input.getPassword(), userRoleView.getPassword())) {
            throw new BusinessException(HttpStatus.CONFLICT,
                    Constants.ERR_CODE_10004,
                    Constants.ERR_TTL_10004,
                    Constants.ERR_MSG_10004);
        }

        var tokenContent = TokenContent.builder()
                .userId(userRoleView.getId())
                .username(userRoleView.getUsername())
                .nama(userRoleView.getNama())
                .role(userRoleView.getRole())
                .accessMenu(new HashSet<>())
                .build();

        Map<String, Object> claims = new HashMap<>();
        claims.put("userId", userRoleView.getId());

        final String accessToken = jwtTokenUtil.generateToken(String.valueOf(userRoleView.getId()), tokenContent, accessTokenExp * 60000);
        String refreshToken = jwtTokenUtil.doGenerateToken(claims, String.valueOf(userRoleView.getId()), refreshTokenExp * 60000);

        if (StringUtils.isBlank(accessToken) || StringUtils.isBlank(refreshToken)) {
            log.error("token null");
            throw new BusinessException("err", "err", "err");
        }

        commonService.saveUserToken(TokenManagement.builder()
                .userId(userRoleView.getId())
                .accessToken(accessToken)
                .refreshToken(refreshToken)
                .build(), accessTokenExp);


    return LoginResponse.builder()
            .accessToken(accessToken)
            .validity(accessTokenExp * 60)
            .refreshToken(refreshToken)
            .username(userRoleView.getUsername())
            .email(userRoleView.getEmail())
            .role(userRoleView.getRole())
            .build();
    }
}
